"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMiddleware = exports.WARRANT_IGNORE_ID = void 0;
const warrant_node_1 = require("@warrantdev/warrant-node");
var warrant_node_2 = require("@warrantdev/warrant-node");
Object.defineProperty(exports, "WARRANT_IGNORE_ID", { enumerable: true, get: function () { return warrant_node_2.WARRANT_IGNORE_ID; } });
function createHasAccessMiddleware(client, getUserId, onAuthorizeFailure) {
    return (objectType, getObjectId, relation) => {
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const objectId = getObjectId(req);
            const userId = getUserId(req);
            if (userId === null) {
                onAuthorizeFailure(req, res);
                return;
            }
            if (yield client.isAuthorized(objectType, objectId, relation, userId.toString())) {
                if (next) {
                    next();
                }
                return;
            }
            onAuthorizeFailure(req, res);
        });
    };
}
function createHasPermissionMiddleware(client, getUserId, onAuthorizeFailure) {
    return (permissionId) => {
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const userId = getUserId(req);
            if (userId === null) {
                onAuthorizeFailure(req, res);
                return;
            }
            if (yield client.hasPermission(permissionId, userId.toString())) {
                if (next) {
                    next();
                }
                return;
            }
            onAuthorizeFailure(req, res);
        });
    };
}
function createMiddleware(config) {
    const client = new warrant_node_1.Client(config.clientKey);
    let onAuthorizeFailure = (req, res) => res.sendStatus(401);
    if (config.onAuthorizeFailure) {
        onAuthorizeFailure = config.onAuthorizeFailure;
    }
    return {
        hasPermission: createHasPermissionMiddleware(client, config.getUserId, onAuthorizeFailure),
        hasAccess: createHasAccessMiddleware(client, config.getUserId, onAuthorizeFailure),
    };
}
exports.createMiddleware = createMiddleware;
